@@ -18,10 +18,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include "config.h"
@@ -146,23 +142,23 @@ static void handle_dahdi_exception(struct e1inp_ts *ts)
 	case DAHDI_EVENT_ALARM:
 		/* we should notify the code that the line is gone */
 		osmo_signal_dispatch(SS_L_INPUT, S_L_INP_LINE_ALARM, &isd);
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_ALARM]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_ALARM));
 		break;
 	case DAHDI_EVENT_NOALARM:
 		/* alarm has gone, we should re-start the SABM requests */
 		osmo_signal_dispatch(SS_L_INPUT, S_L_INP_LINE_NOALARM, &isd);
 		break;
 	case DAHDI_EVENT_ABORT:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_HDLC_ABORT]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_HDLC_ABORT));
 		break;
 	case DAHDI_EVENT_OVERRUN:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_HDLC_OVERR]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_HDLC_OVERR));
 		break;
 	case DAHDI_EVENT_BADFCS:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_HDLC_BADFCS]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_HDLC_BADFCS));
 		break;
 	case DAHDI_EVENT_REMOVED:
-		rate_ctr_inc(&line->rate_ctr->ctr[E1I_CTR_REMOVED]);
+		rate_ctr_inc(rate_ctr_group_get_ctr(line->rate_ctr, E1I_CTR_REMOVED));
 		break;
 	}
 }
@@ -199,12 +195,13 @@ static int ts_want_write(struct e1inp_ts *e1i_ts)
 	/* We never include the DAHDI B-Channel FD into the
 	 * writeset, since it doesn't support poll() based
 	 * write flow control */
-	if (e1i_ts->type == E1INP_TS_TYPE_TRAU) {
+	if (e1i_ts->type == E1INP_TS_TYPE_TRAU ||
+	    e1i_ts->type == E1INP_TS_TYPE_I460) {
 		LOGPITS(e1i_ts, DLINP, LOGL_DEBUG, "Trying to write TRAU ts\n");
 		return 0;
 	}
 
-	e1i_ts->driver.dahdi.fd.when |= BSC_FD_WRITE;
+	osmo_fd_write_enable(&e1i_ts->driver.dahdi.fd);
 
 	return 0;
 }
@@ -250,7 +247,7 @@ static int handle_ts1_write(struct osmo_fd *bfd)
 	struct e1inp_sign_link *sign_link;
 	struct msgb *msg;
 
-	bfd->when &= ~BSC_FD_WRITE;
+	osmo_fd_write_disable(bfd);
 
 	/* get the next msg for this timeslot */
 	msg = e1inp_tx_ts(e1i_ts, &sign_link);
@@ -472,38 +469,39 @@ static int dahdi_fd_cb(struct osmo_fd *bfd, unsigned int what)
 
 	switch (e1i_ts->type) {
 	case E1INP_TS_TYPE_SIGN:
-		if (what & BSC_FD_EXCEPT)
+		if (what & OSMO_FD_EXCEPT)
 			handle_dahdi_exception(e1i_ts);
-		if (what & BSC_FD_READ)
+		if (what & OSMO_FD_READ)
 			rc = handle_ts1_read(bfd);
-		if (what & BSC_FD_WRITE)
+		if (what & OSMO_FD_WRITE)
 			rc = handle_ts1_write(bfd);
 		break;
 	case E1INP_TS_TYPE_HDLC:
-		if (what & BSC_FD_EXCEPT)
+		if (what & OSMO_FD_EXCEPT)
 			handle_dahdi_exception(e1i_ts);
-		if (what & BSC_FD_READ)
+		if (what & OSMO_FD_READ)
 			rc = handle_hdlc_read(bfd);
-		if (what & BSC_FD_WRITE)
+		if (what & OSMO_FD_WRITE)
 			handle_hdlc_write(bfd);
 		break;
 	case E1INP_TS_TYPE_TRAU:
-		if (what & BSC_FD_EXCEPT)
+		if (what & OSMO_FD_EXCEPT)
 			handle_dahdi_exception(e1i_ts);
-		if (what & BSC_FD_READ)
+		if (what & OSMO_FD_READ)
 			rc = handle_tsX_read(bfd);
-		if (what & BSC_FD_WRITE)
+		if (what & OSMO_FD_WRITE)
 			rc = handle_tsX_write(bfd);
 		/* We never include the DAHDI B-Channel FD into the
 		 * writeset, since it doesn't support poll() based
 		 * write flow control */
 		break;
+	case E1INP_TS_TYPE_I460:
 	case E1INP_TS_TYPE_RAW:
-		if (what & BSC_FD_EXCEPT)
+		if (what & OSMO_FD_EXCEPT)
 			handle_dahdi_exception(e1i_ts);
-		if (what & BSC_FD_READ)
+		if (what & OSMO_FD_READ)
 			rc = handle_ts_raw_read(bfd);
-		if (what & BSC_FD_WRITE)
+		if (what & OSMO_FD_WRITE)
 			rc = handle_ts_raw_write(bfd);
 		/* We never include the DAHDI B-Channel FD into the
 		 * writeset, since it doesn't support poll() based
@@ -526,7 +524,7 @@ static void dahdi_vty_show(struct vty *vty, struct e1inp_line *line)
 
 	scfg = span_cfgs[line->port_nr];
 	if (!scfg) {
-		vty_out(vty, "DAHDI Span %u non-existant%s",
+		vty_out(vty, "DAHDI Span %u non-existent%s",
 			line->port_nr+1, VTY_NEWLINE);
 		return;
 	}
@@ -643,10 +641,6 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 		 * board will be 32 */
 		dev_nr = scfg->chan_base + idx;
 
-		bfd->data = line;
-		bfd->priv_nr = ts;
-		bfd->cb = dahdi_fd_cb;
-
 		switch (e1i_ts->type) {
 		case E1INP_TS_TYPE_NONE:
 			/* close/release LAPD instance, if any */
@@ -665,7 +659,7 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 				bfd->fd = dahdi_open_slot(dev_nr);
 			if (bfd->fd < 0)
 				return -EIO;
-			bfd->when = BSC_FD_READ | BSC_FD_EXCEPT;
+			bfd->when = OSMO_FD_READ | OSMO_FD_EXCEPT;
 			ret = dahdi_set_bufinfo(bfd->fd, 1);
 			if (ret < 0)
 				return ret;
@@ -683,12 +677,13 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 				bfd->fd = dahdi_open_slot(dev_nr);
 			if (bfd->fd < 0)
 				return -EIO;
-			bfd->when = BSC_FD_READ | BSC_FD_EXCEPT;
+			bfd->when = OSMO_FD_READ | OSMO_FD_EXCEPT;
 			ret = dahdi_set_bufinfo(bfd->fd, 1);
 			if (ret < 0)
 				return ret;
 			break;
 		case E1INP_TS_TYPE_TRAU:
+		case E1INP_TS_TYPE_I460:
 		case E1INP_TS_TYPE_RAW:
 			/* close/release LAPD instance, if any */
 			if (e1i_ts->lapd) {
@@ -705,13 +700,14 @@ static int dahdi_e1_setup(struct e1inp_line *line)
 			/* We never include the DAHDI B-Channel FD into the
 			 * writeset, since it doesn't support poll() based
 			 * write flow control */
-			bfd->when = BSC_FD_READ | BSC_FD_EXCEPT;// | BSC_FD_WRITE;
+			bfd->when = OSMO_FD_READ | OSMO_FD_EXCEPT;// | OSMO_FD_WRITE;
 			break;
 		}
 
 		if (bfd->fd < 0)
 			return bfd->fd;
 
+		osmo_fd_setup(bfd, bfd->fd, bfd->when, dahdi_fd_cb, line, ts);
 		ret = osmo_fd_register(bfd);
 		if (ret < 0) {
 			LOGPITS(e1i_ts, DLINP, LOGL_ERROR, "could not register FD: %s\n", strerror(ret));
