@@ -1,6 +1,6 @@
 /* OpenBSC Abis input driver for ip.access */
 
-/* (C) 2009 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2009-2021 by Harald Welte <laforge@gnumonks.org>
  * (C) 2010 by Holger Hans Peter Freyther
  * (C) 2010 by On-Waves
  *
@@ -50,6 +50,10 @@
 #include <osmocom/abis/ipa.h>
 #include <osmocom/core/backtrace.h>
 #include <osmocom/gsm/ipa.h>
+#include <osmocom/core/stats_tcp.h>
+
+/* global parameters of IPA input driver */
+struct ipa_pars g_e1inp_ipaccess_pars;
 
 static void *tall_ipa_ctx;
 
@@ -88,19 +92,31 @@ static int ipaccess_drop(struct osmo_fd *bfd, struct e1inp_line *line)
 
 	/* Error case: we did not see any ID_RESP yet for this socket. */
 	if (bfd->fd != -1) {
-		LOGP(DLINP, LOGL_ERROR, "Forcing socket shutdown with "
-					"no signal link set\n");
+		LOGPITS(e1i_ts, DLINP, LOGL_NOTICE, "Forcing socket shutdown\n");
 		osmo_fd_unregister(bfd);
 		close(bfd->fd);
 		bfd->fd = -1;
-		/* This is BSC code, ipaccess_drop() is only called for
-		   accepted() sockets, hence the bfd holds a reference to
-		   e1inp_line in ->data that needs to be released */
-		OSMO_ASSERT(bfd->data == line);
-		bfd->data = NULL;
-		e1inp_line_put2(line, "ipa_bfd");
-
+		switch (line->ops->cfg.ipa.role) {
+		case E1INP_LINE_R_BSC:
+			/* This is BSC code, ipaccess_drop() is only called for
+			   accepted() sockets, hence the bfd holds a reference to
+			   e1inp_line in ->data that needs to be released */
+			OSMO_ASSERT(bfd->data == line);
+			bfd->data = NULL;
+			e1inp_line_put2(line, "ipa_bfd");
+			break;
+		case E1INP_LINE_R_BTS:
+			/* BTS code: bfd->data contains pointer to struct
+			 * ipa_client_conn. Leave it alive so it reconnects.
+			 */
+			break;
+		default:
+			break;
+		}
 		ret = -ENOENT;
+	} else {
+		LOGPITS(e1i_ts, DLINP, LOGL_ERROR,
+			"Forcing socket shutdown with no signal link set\n");
 	}
 
 	msgb_free(e1i_ts->pending_msg);
@@ -280,6 +296,7 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 			struct e1inp_ts *ts;
 			struct osmo_fd *newbfd;
 			struct e1inp_line *new_line;
+			char tcp_stat_name[64];
 
 			sign_link =
 				line->ops->sign_link_up(&unit_data, line,
@@ -326,6 +343,9 @@ static int ipaccess_rcvmsg(struct e1inp_line *line, struct msgb *msg,
 				     "could not register FD\n");
 				goto err;
 			}
+			snprintf(tcp_stat_name, sizeof(tcp_stat_name), "site.%u.bts.%u.ipa-rsl.%u",
+				unit_data.site_id, unit_data.bts_id, unit_data.trx_id);
+			osmo_stats_tcp_osmo_fd_register(newbfd, tcp_stat_name);
 
 			e1i_ts = ipaccess_line_ts(newbfd, new_line);
 			ipaccess_bsc_keepalive_fsm_alloc(e1i_ts, newbfd, "rsl_bsc_to_bts");
@@ -422,7 +442,7 @@ err:
 
 static int ts_want_write(struct e1inp_ts *e1i_ts)
 {
-	e1i_ts->driver.ipaccess.fd.when |= BSC_FD_WRITE;
+	osmo_fd_write_enable(&e1i_ts->driver.ipaccess.fd);
 
 	return 0;
 }
@@ -457,12 +477,24 @@ static void ipaccess_close(struct e1inp_sign_link *sign_link)
 	}
 }
 
+static bool e1i_ts_has_pending_tx_msgs(struct e1inp_ts *e1i_ts)
+{
+	struct e1inp_sign_link *link;
+	llist_for_each_entry(link, &e1i_ts->sign.sign_links, list) {
+		if (!llist_empty(&link->tx_list)) {
+			return true;
+		}
+	}
+	return false;
+}
+
 static void timeout_ts1_write(void *data)
 {
 	struct e1inp_ts *e1i_ts = (struct e1inp_ts *)data;
 
 	/* trigger write of ts1, due to tx delay timer */
-	ts_want_write(e1i_ts);
+	if (e1i_ts_has_pending_tx_msgs(e1i_ts))
+		ts_want_write(e1i_ts);
 }
 
 static int __handle_ts1_write(struct osmo_fd *bfd, struct e1inp_line *line)
@@ -474,12 +506,12 @@ static int __handle_ts1_write(struct osmo_fd *bfd, struct e1inp_line *line)
 	int ret;
 
 	e1i_ts = ipaccess_line_ts(bfd, line);
-	bfd->when &= ~BSC_FD_WRITE;
 
 	/* get the next msg for this timeslot */
 	msg = e1inp_tx_ts(e1i_ts, &sign_link);
 	if (!msg) {
 		/* no message after tx delay timer */
+		osmo_fd_write_disable(bfd);
 		return 0;
 	}
 
@@ -489,7 +521,7 @@ static int __handle_ts1_write(struct osmo_fd *bfd, struct e1inp_line *line)
 	case E1INP_SIGN_OSMO:
 		break;
 	default:
-		bfd->when |= BSC_FD_WRITE; /* come back for more msg */
+		/* leave WRITE flag enabled, come back for more msg */
 		ret = -EINVAL;
 		goto out;
 	}
@@ -497,22 +529,29 @@ static int __handle_ts1_write(struct osmo_fd *bfd, struct e1inp_line *line)
 	msg->l2h = msg->data;
 	ipa_prepend_header(msg, sign_link->tei);
 
-	DEBUGP(DLMI, "TX %u: %s\n", ts_nr, osmo_hexdump(msg->l2h, msgb_l2len(msg)));
+	LOGPITS(e1i_ts, DLMI, LOGL_DEBUG, "TX %u: %s\n", ts_nr,
+		osmo_hexdump(msg->l2h, msgb_l2len(msg)));
 
 	ret = send(bfd->fd, msg->data, msg->len, 0);
 	if (ret != msg->len) {
-		LOGP(DLINP, LOGL_ERROR, "failed to send A-bis IPA signalling "
+		LOGPITS(e1i_ts, DLINP, LOGL_ERROR, "failed to send A-bis IPA signalling "
 			"message. Reason: %s\n", strerror(errno));
 		goto err;
 	}
 
-	/* set tx delay timer for next event */
-	osmo_timer_setup(&e1i_ts->sign.tx_timer, timeout_ts1_write, e1i_ts);
-
-	/* Reducing this might break the nanoBTS 900 init. */
-	osmo_timer_schedule(&e1i_ts->sign.tx_timer, 0, e1i_ts->sign.delay);
-
+	/* this is some ancient code that apparently exists to slow down writes towards
+	 * some even more ancient nanoBTS 900 units. See git commit
+	 * d49fc5ae24fc9d44d2b284392ab619cc7a69a876 of openbsc.git (now osmo-bsc.git) */
+	if (e1i_ts->sign.delay) {
+		osmo_fd_write_disable(bfd);
+		/* set tx delay timer for next event */
+		osmo_timer_setup(&e1i_ts->sign.tx_timer, timeout_ts1_write, e1i_ts);
+		osmo_timer_schedule(&e1i_ts->sign.tx_timer, 0, e1i_ts->sign.delay);
+	} else {
 out:
+		if (!e1i_ts_has_pending_tx_msgs(e1i_ts))
+			osmo_fd_write_disable(bfd);
+	}
 	msgb_free(msg);
 	return ret;
 err:
@@ -540,9 +579,9 @@ int ipaccess_fd_cb(struct osmo_fd *bfd, unsigned int what)
 {
 	int rc = 0;
 
-	if (what & BSC_FD_READ)
+	if (what & OSMO_FD_READ)
 		rc = handle_ts1_read(bfd);
-	if (rc != -EBADF && (what & BSC_FD_WRITE))
+	if (rc != -EBADF && (what & OSMO_FD_WRITE))
 		rc = handle_ts1_write(bfd);
 
 	return rc;
@@ -649,12 +688,13 @@ static int ipaccess_bsc_oml_cb(struct ipa_server_link *link, int fd)
 	e1i_ts = e1inp_line_ipa_oml_ts(line);
 
 	bfd = &e1i_ts->driver.ipaccess.fd;
-	osmo_fd_setup(bfd, fd, BSC_FD_READ, ipaccess_fd_cb, line, E1INP_SIGN_OML);
+	osmo_fd_setup(bfd, fd, OSMO_FD_READ, ipaccess_fd_cb, line, E1INP_SIGN_OML);
 	ret = osmo_fd_register(bfd);
 	if (ret < 0) {
 		LOGP(DLINP, LOGL_ERROR, "could not register FD\n");
 		goto err_line;
 	}
+	osmo_stats_tcp_osmo_fd_register(bfd, "ipa-oml");
 
 	update_fd_settings(line, bfd->fd);
 
@@ -702,12 +742,14 @@ static int ipaccess_bsc_rsl_cb(struct ipa_server_link *link, int fd)
 	e1i_ts = e1inp_line_ipa_rsl_ts(line, 0);
 
 	bfd = &e1i_ts->driver.ipaccess.fd;
-	osmo_fd_setup(bfd, fd, BSC_FD_READ, ipaccess_fd_cb, line, E1INP_SIGN_RSL);
+	osmo_fd_setup(bfd, fd, OSMO_FD_READ, ipaccess_fd_cb, line, E1INP_SIGN_RSL);
 	ret = osmo_fd_register(bfd);
 	if (ret < 0) {
 		LOGP(DLINP, LOGL_ERROR, "could not register FD\n");
 		goto err_line;
 	}
+	osmo_stats_tcp_osmo_fd_register(bfd, "ipa-rsl");
+
 	/* Request ID. FIXME: request LOCATION, HW/SW VErsion, Unit Name, Serno */
 	ret = ipa_ccm_send_id_req(bfd->fd);
 	if (ret < 0) {
@@ -989,6 +1031,7 @@ err:
 
 struct ipaccess_line {
 	bool line_already_initialized;
+	struct ipa_client_conn *ipa_cli[NUM_E1_TS]; /* 0=OML, 1+N=TRX_N */
 };
 
 static int ipaccess_line_update(struct e1inp_line *line)
@@ -1005,14 +1048,11 @@ static int ipaccess_line_update(struct e1inp_line *line)
 	}
 	il = line->driver_data;
 
-	/* We only initialize this line once. */
-	if (il->line_already_initialized)
-		return 0;
-
-	il->line_already_initialized = true;
-
 	switch(line->ops->cfg.ipa.role) {
 	case E1INP_LINE_R_BSC: {
+		/* We only initialize this line once. */
+		if (il->line_already_initialized)
+			return 0;
 		struct ipa_server_link *oml_link, *rsl_link;
 		const char *ipa = e1inp_ipa_get_bind_addr();
 
@@ -1028,6 +1068,8 @@ static int ipaccess_line_update(struct e1inp_line *line)
 				"BSC link: %s\n", strerror(errno));
 			return -ENOMEM;
 		}
+		oml_link->dscp = g_e1inp_ipaccess_pars.oml.dscp;
+		oml_link->priority = g_e1inp_ipaccess_pars.oml.priority;
 		if (ipa_server_link_open(oml_link) < 0) {
 			LOGP(DLINP, LOGL_ERROR, "cannot open OML BSC link: %s\n",
 				strerror(errno));
@@ -1042,6 +1084,8 @@ static int ipaccess_line_update(struct e1inp_line *line)
 				"BSC link: %s\n", strerror(errno));
 			return -ENOMEM;
 		}
+		rsl_link->dscp = g_e1inp_ipaccess_pars.rsl.dscp;
+		rsl_link->priority = g_e1inp_ipaccess_pars.rsl.priority;
 		if (ipa_server_link_open(rsl_link) < 0) {
 			LOGP(DLINP, LOGL_ERROR, "cannot open RSL BSC link: %s\n",
 				strerror(errno));
@@ -1059,6 +1103,13 @@ static int ipaccess_line_update(struct e1inp_line *line)
 		     "OML connecting to %s:%u\n", line->ops->cfg.ipa.addr,
 		     IPA_TCP_PORT_OML);
 
+		/* Drop previous line */
+		if (il->ipa_cli[0]) {
+			ipa_client_conn_close(il->ipa_cli[0]);
+			ipa_client_conn_destroy(il->ipa_cli[0]);
+			il->ipa_cli[0] = NULL;
+		}
+
 		link = ipa_client_conn_create2(tall_ipa_ctx,
 					      e1inp_line_ipa_oml_ts(line),
 					      E1INP_SIGN_OML,
@@ -1074,6 +1125,8 @@ static int ipaccess_line_update(struct e1inp_line *line)
 				"BTS link: %s\n", strerror(errno));
 			return -ENOMEM;
 		}
+		link->dscp = g_e1inp_ipaccess_pars.oml.dscp;
+		link->priority = g_e1inp_ipaccess_pars.oml.priority;
 		if (ipa_client_conn_open(link) < 0) {
 			LOGP(DLINP, LOGL_ERROR, "cannot open OML BTS link: %s\n",
 				strerror(errno));
@@ -1084,12 +1137,15 @@ static int ipaccess_line_update(struct e1inp_line *line)
 
 		e1i_ts = e1inp_line_ipa_oml_ts(line);
 		ipaccess_bts_keepalive_fsm_alloc(e1i_ts, link, "oml_bts_to_bsc");
+		il->ipa_cli[0] = link;
 		ret = 0;
 		break;
 	}
 	default:
 		break;
 	}
+
+	il->line_already_initialized = true;
 	return ret;
 }
 
@@ -1107,6 +1163,7 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 {
 	struct ipa_client_conn *rsl_link;
 	struct e1inp_ts *e1i_ts = e1inp_line_ipa_rsl_ts(line, trx_nr);
+	struct ipaccess_line *il;
 
 	if (E1INP_SIGN_RSL+trx_nr-1 >= NUM_E1_TS) {
 		LOGP(DLINP, LOGL_ERROR, "cannot create RSL BTS link: "
@@ -1114,6 +1171,17 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 		return -EINVAL;
 	}
 
+	if (!line->driver_data)
+		line->driver_data = talloc_zero(line, struct ipaccess_line);
+	il = line->driver_data;
+
+	/* Drop previous line */
+	if (il->ipa_cli[1 + trx_nr]) {
+		ipa_client_conn_close(il->ipa_cli[1 + trx_nr]);
+		ipa_client_conn_destroy(il->ipa_cli[1 + trx_nr]);
+		il->ipa_cli[1 + trx_nr] = NULL;
+	}
+
 	rsl_link = ipa_client_conn_create2(tall_ipa_ctx,
 					  e1inp_line_ipa_rsl_ts(line, trx_nr),
 					  E1INP_SIGN_RSL+trx_nr,
@@ -1128,6 +1196,8 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 			"BTS link: %s\n", strerror(errno));
 		return -ENOMEM;
 	}
+	rsl_link->dscp = g_e1inp_ipaccess_pars.rsl.dscp;
+	rsl_link->priority = g_e1inp_ipaccess_pars.rsl.priority;
 	if (ipa_client_conn_open(rsl_link) < 0) {
 		LOGP(DLINP, LOGL_ERROR, "cannot open RSL BTS link: %s\n",
 			strerror(errno));
@@ -1135,8 +1205,8 @@ int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 		ipa_client_conn_destroy(rsl_link);
 		return -EIO;
 	}
-
 	ipaccess_bts_keepalive_fsm_alloc(e1i_ts, rsl_link, "rsl_bts_to_bsc");
+	il->ipa_cli[1 + trx_nr] = rsl_link;
 	return 0;
 }
 
