@@ -1,4 +1,4 @@
-/* (C) 2011 by Harald Welte <laforge@gnumonks.org>
+/* (C) 2011-2021 by Harald Welte <laforge@gnumonks.org>
  * (C) 2011 by On-Waves e.h.f
  * All Rights Reserved
  *
@@ -14,10 +14,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \file osmo_ortp.c
@@ -33,6 +29,7 @@
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/utils.h>
 #include <osmocom/core/select.h>
+#include <osmocom/core/socket.h>
 #include <osmocom/trau/osmo_ortp.h>
 
 #include <ortp/ortp.h>
@@ -141,8 +138,8 @@ static void ortp_sig_cb_ssrc(RtpSession *rs, void *data)
 	uint32_t ssrc = rtp_session_get_recv_ssrc(rs);
 
 	LOGP(DLMIB, LOGL_INFO,
-	     "osmo-ortp(%d): ssrc_changed to 0x%08x, resetting\n", port, ssrc);
-	rtp_session_reset(rs);
+	     "osmo-ortp(%d): ssrc_changed to 0x%08x, resyncing\n", port, ssrc);
+	rtp_session_resync(rs);
 }
 
 static void ortp_sig_cb_pt(RtpSession *rs, void *data)
@@ -202,8 +199,10 @@ int osmo_rtp_socket_poll(struct osmo_rtp_socket *rs)
 	if (recv_with_cb(rs))
 		return 1;
 
-	LOGP(DLMIB, LOGL_INFO, "osmo_rtp_socket_poll(%u): ERROR!\n",
-	     rs->rx_user_ts);
+	/* this happens every time we miss an incoming RTP frame, which is quite common
+	 * when a voice channel is first activated, or also in case of packet loss.
+	 * See also https://osmocom.org/issues/4464 */
+	LOGP(DLMIB, LOGL_DEBUG, "osmo_rtp_socket_poll(%u): No message received\n", rs->rx_user_ts);
 	return 0;
 }
 
@@ -213,10 +212,10 @@ static int osmo_rtp_fd_cb(struct osmo_fd *fd, unsigned int what)
 {
 	struct osmo_rtp_socket *rs = fd->data;
 
-	if (what & BSC_FD_READ) {
+	if (what & OSMO_FD_READ) {
 		/* in polling mode, we don't want to be called here */
 		if (rs->flags & OSMO_RTP_F_POLL) {
-			fd->when &= ~BSC_FD_READ;
+			osmo_fd_read_disable(fd);
 			return 0;
 		}
 		if (!recv_with_cb(rs))
@@ -224,7 +223,7 @@ static int osmo_rtp_fd_cb(struct osmo_fd *fd, unsigned int what)
 			     rs->rx_user_ts);
 		rs->rx_user_ts += 160;
 	}
-	/* writing is not queued at the moment, so BSC_FD_WRITE
+	/* writing is not queued at the moment, so OSMO_FD_WRITE
 	 * shouldn't occur */
 	return 0;
 }
@@ -247,12 +246,8 @@ static int osmo_rtp_socket_fdreg(struct osmo_rtp_socket *rs)
 {
 	int rc;
 
-	rs->rtp_bfd.fd = rtp_session_get_rtp_socket(rs->sess);
-	rs->rtcp_bfd.fd = rtp_session_get_rtcp_socket(rs->sess);
-	rs->rtp_bfd.when = rs->rtcp_bfd.when = BSC_FD_READ;
-	rs->rtp_bfd.data = rs->rtcp_bfd.data = rs;
-	rs->rtp_bfd.cb = osmo_rtp_fd_cb;
-	rs->rtcp_bfd.cb = osmo_rtcp_fd_cb;
+	osmo_fd_setup(&rs->rtp_bfd, rtp_session_get_rtp_socket(rs->sess), OSMO_FD_READ, osmo_rtp_fd_cb, rs, 0);
+	osmo_fd_setup(&rs->rtcp_bfd, rtp_session_get_rtcp_socket(rs->sess), OSMO_FD_READ, osmo_rtcp_fd_cb, rs, 0);
 
 	rc = osmo_fd_register(&rs->rtp_bfd);
 	if (rc < 0)
@@ -594,6 +589,21 @@ int osmo_rtp_socket_set_dscp(struct osmo_rtp_socket *rs, int dscp)
 	return rtp_session_set_dscp(rs->sess, dscp);
 }
 
+/*! \brief Set the socket priority for outgoing RTP packets
+ *  \param[in] rs OsmoRTP socket
+ *  \param[in] prio socket priority
+ *  \returns 0 on success, < 0 otherwise
+ */
+int osmo_rtp_socket_set_priority(struct osmo_rtp_socket *rs, uint8_t prio)
+{
+	int rc;
+
+	rc = osmo_sock_set_priority(rs->rtp_bfd.fd, prio);
+	if (rc < 0)
+		return rc;
+	return osmo_sock_set_priority(rs->rtcp_bfd.fd, prio);
+}
+
 /*! \brief completely close the RTP socket and release all resources
  *  \param[in] rs OsmoRTP socket to be released
  *  \returns 0 on success
